/* --COPYRIGHT--,BSD
* Copyright (c) 2017, Texas Instruments Incorporated
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
* *  Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*
* *  Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
*
* *  Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
* OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
* --/COPYRIGHT--*/
/*******************************************************************************
* MSP432 GPIO - Input Interrupt
*
* Description: This example demonstrates a very simple use case of the
* DriverLib GPIO APIs. P1.1 (which has a switch connected to it) is configured
* as an input with interrupts enabled and P1.0 (which has an LED connected)
* is configured as an output. When the switch is pressed, the LED output
* is toggled.
*
*                MSP432P401
*             ------------------
*         /|\|                  |
*          | |                  |
*          --|RST         P1.0  |---> P1.0 LED
*            |                  |
*            |            P1.1  |<--Toggle Switch
*            |                  |
*            |                  |
*
******************************************************************************/
/* DriverLib Includes */
#include <ti/devices/msp432p4xx/driverlib/driverlib.h>

/* Standard Includes */
#include <stdint.h>
#include <stdbool.h>

#define PORT_BUTTON_INTERRUPT (GPIO_PORT_P6)
#define PIN_BUTTON_INTERRUPT  (GPIO_PIN7)

#define BSL_PARAM (0xFC48FFFF)
//#define BSL_PARAM (0xFC48DFFF)

uint8_t flag_interrupt = 0;

int main(void)
{
    /* Halting the Watchdog */
    MAP_WDT_A_holdTimer();

    /* Configuring P1.0 */
    MAP_GPIO_setAsOutputPin(GPIO_PORT_P1, GPIO_PIN0);

    /* Configuring P6.7 as an input and enabling interrupts */
    MAP_GPIO_setAsInputPinWithPullDownResistor(PORT_BUTTON_INTERRUPT, PIN_BUTTON_INTERRUPT);
    MAP_GPIO_clearInterruptFlag(PORT_BUTTON_INTERRUPT, PIN_BUTTON_INTERRUPT);
    MAP_GPIO_enableInterrupt(PORT_BUTTON_INTERRUPT, PIN_BUTTON_INTERRUPT);
    MAP_Interrupt_enableInterrupt(INT_PORT6);

    /* Enabling SRAM Bank Retention */
    MAP_SysCtl_enableSRAMBankRetention(SYSCTL_SRAM_BANK1);

    /* Enabling MASTER interrupts */
    MAP_Interrupt_enableMaster();

    flag_interrupt = 0;

    /* Going to LPM3 */
    while (1)
    {
        if(1==flag_interrupt)
        {
            MAP_Interrupt_disableMaster();
            NVIC->ICER[0] = 0xFFFF;
            NVIC->ICPR[0] = 0xFFFF;
            NVIC->ICER[1] = 0xFFFF;
            NVIC->ICPR[1] = 0xFFFF;
            ((void (*)())BSL_ENTRY_FUNCTION)((uint32_t)BSL_PARAM); // Call the BSL with given BSL parameters
        }
    }
}

/* GPIO ISR */
void PORT6_IRQHandler(void)
{
    uint32_t status;

    status = MAP_GPIO_getEnabledInterruptStatus(PORT_BUTTON_INTERRUPT);
    MAP_GPIO_clearInterruptFlag(PORT_BUTTON_INTERRUPT, status);

    /* Toggling the output on the LED */
    if(status & PIN_BUTTON_INTERRUPT)
    {
        MAP_GPIO_toggleOutputOnPin(GPIO_PORT_P1, GPIO_PIN0);

        flag_interrupt = 1;
    }

    /* Calling BSL */


}
